@@ -12,10 +12,12 @@
 #' @references Viole, F. and Nawrocki, D. (2013) "Nonlinear Nonparametric Statistics: Using Partial Moments"
 #' \url{https://www.amazon.com/dp/1490523995/ref=cm_sw_su_dp}
 #' @examples
+#' \dontrun{
 #' set.seed(123)
 #' x <- rnorm(100) ; y<-rnorm(100)
 #' A <- cbind(x, y)
 #' NNS.norm(A)
+#' }
 #' @export
 
 NNS.norm <- function(X,
@@ -29,7 +31,7 @@ NNS.norm <- function(X,
 
   X <- apply(X, 2, unlist)
 
-  m  <- colMeans(X)
+  m  <- Rfast::colmeans(X)
   m[m==0] <- 1e-10
   RG <- m %o% (1 / m)
 
